# ITECS Dusk Macros

Macros extracted from various ITECS Web Team applications, for use in Laravel Dusk end-to-end test cases.

Where noted, macros are specific to Vuetify 2.x user interfaces.

## Install

### Requirements

This package is aimed at users of the Laravel Dusk end-to-end Browser-based testing package.

The following version of PHP is required:

* PHP >= 7.2.0

For developmental purpose, you need have at least the following version of PHPUnit:

* PHPUnit >= 7.5.10

### Configuration 

Add the package as a project development dependency. First, ensure that the local packages source is declared in your composer.json:

```
{
    "repositories": [
        {
            "type": "vcs",
            "url": "git@github.ncsu.edu:itecs/itecsduskmacros.git"
        }
    ]
}

```

Then install the package:

```
composer require ncsu/itecsduskmacros
```

In `DuskServiceProvider.php`, add the following code at top:

```use NCSU\ItecsDuskMacros\DuskMacros;```

Finally in your `boot()` function, register the Dusk macros:

```
    /**
     * Bootstrap services.
     *
     * @return void
     */
    public function boot()
    {
        DuskMacros::register();
    }
```

After that, your Dusk tests should have several new macros available.

## Usage

| Macro         | Description                                                         |
|---------------|---------------------------------------------------------------------|
|consoleLogs| return the browser console log as an array of messages              |
|consoleLogErrors| return browser console logs that are not INFO type messages         |
|diagnosticOutput| send output to STDERR while test is running                         |
|dumpConsoleLogs| send consoleLogs() array to STDOUT while test is running            |
|eraseField| simulate backspacing over an input field                            |
|fillHiddenField| place data into a hidden form field                                 |
|optionXPathByText| return path to Vuetify Option by its text content                   |
|sendSpaceToCheckbox| simulate checking a box with the spacebar                           |
|vAutocompleteOption| select a Vuetify autocomplete option based on its text content      |
|vCheckbox| click a Vuetify checkbox based on a dusk, css, id, or xpath selector |
|vSelectOption| select a Vuetify select option based on its text content            |
|vToggle| click a Vuetify toggle based on a dusk, css, id or xpath selector   |
|waitAndClickOn| wait for element to appear then click on it                         |

## Contributing

To propose a new macro to this package, open a PR.

Macros are added to the toArray method within the class DuskMacros. 
Each macro consists of a string name, and a closure to be executed within the context of a Dusk Browser object.

- _Provide a unit test case to verify the soundness of the macro closure. 
- _Please keep macros in alphabetic order._

## Testing

Unit test cases should be written for each macro to ensure that it includes the minimum necessary dependencies.

_TBD: a good-enough Browser mock/spy class to avoid having to include the entire Dusk framework._

To exercise the tests just run PHPUnit:

```
phpunit
```

## License

The MIT License. Please see [License File](./LICENSE) for more information.