<?php

namespace NCSU\ItecsDuskMacros\Tests;

use PHPUnit\Framework\TestCase;
use NCSU\ItecsDuskMacros\DuskMacros;
use DuskBrowserMock;

/**
 * Prevent setting the class alias for all test suites
 * @runTestsInSeparateProcesses
 * @preserveGlobalState disabled
 */
final class DuskMacrosTest extends TestCase
{
    public function setUp(): void
    {
        // set up test to mock the laravel class
        // see https://ericdraken.com/phpunit-mock-hard-dependencies-aliases/
        class_alias(
            'NCSU\ItecsDuskMacros\Tests\DuskBrowserMock',
            'Laravel\Dusk\Browser',
            true
        );

        class_alias(
            'NCSU\ItecsDuskMacros\Tests\DuskBrowserMock',
            'Facebook\WebDriver\WebDriverBy',
            true
        );

        class_alias(
            'NCSU\ItecsDuskMacros\Tests\DuskBrowserMock',
            'Facebook\WebDriver\WebDriverKeys',
            true
        );
    }

    /** @test */
    public function it_can_enumerate_an_array_of_dusk_macros_by_name(): void
    {
        $listOfMacros = DuskMacros::toArray();
        $this->assertIsArray($listOfMacros);
        $this->assertNotEmpty($listOfMacros);
    }

    /** @test */
    public function it_can_register_macros_with_dusk(): void
    {
        $count = count(DuskMacros::toArray());

        // invoke the dusk macros registering function
        DuskMacros::register();

        // check that the Browser Macro interface got called once for each macro
        $this->assertCount($count, \Laravel\Dusk\Browser::$macros);
    }

    /** @test */
    public function it_has_macros_that_do_not_have_unresolved_dependencies(): void
    {
        $browser = new \Laravel\Dusk\Browser();
        DuskMacros::register();

        $result = $browser->consoleLogs();
        $this->assertIsArray($result);

        $result = $browser->consoleLogErrors();
        $this->assertIsArray($result);

        // $browser->diagnosticOutput(['whatev']);
        $browser->dumpConsoleLogs();

        $browser->eraseField('@flamboyent');
        $browser->script('x=1;');
        $browser->fillHiddenField('@somefield', 'some value');

        $path = $browser->optionXPathByText('text');
        $this->assertNotEmpty($path);

        $browser->sendSpaceToCheckbox('@checkbox');

        $browser->vAutocompleteOption('@autocompl', 'optiontext');
        $browser->vCheckbox('@vCheckbox');
        $browser->vSelectOption('@vSelect', 'optiontext');
        $browser->vToggle('@aToggle');
        $browser->waitAndClickOn('@somethin');
    }
}
